@@ -24,7 +24,8 @@
 using namespace std;
 
 AttCompiler::AttCompiler() :
-starting_state(0)
+starting_state(0),
+default_weight(0.0000)
 {
 }
 
@@ -32,31 +33,30 @@ AttCompiler::~AttCompiler()
 {
 }
 
-void 
-AttCompiler::clear() 
+void
+AttCompiler::clear()
 {
-  for (map<int, AttNode*>::const_iterator it = graph.begin(); it != graph.end();
-      ++it) 
+  for (auto& it : graph)
   {
-    delete it->second;
+    delete it.second;
   }
   graph.clear();
   alphabet = Alphabet();
 }
 
-/** 
+/**
  * Converts symbols like @0@ to epsilon, @_SPACE_@ to space, etc.
  * @todo Are there other special symbols? If so, add them, and maybe use a map
  *       for conversion?
  */
-void 
-AttCompiler::convert_hfst(wstring& symbol) 
+void
+AttCompiler::convert_hfst(wstring& symbol)
 {
-  if (symbol == L"@0@" || symbol == L"ε") 
+  if (symbol == L"@0@" || symbol == L"ε")
   {
     symbol = L"";
-  } 
-  else if (symbol == L"@_SPACE_@") 
+  }
+  else if (symbol == L"@_SPACE_@")
   {
     symbol = L" ";
   }
@@ -74,8 +74,8 @@ AttCompiler::is_word_punct(wchar_t symbol)
   {
     return true;
   }
- 
-  return false; 
+
+  return false;
 }
 
 /**
@@ -87,8 +87,8 @@ AttCompiler::is_word_punct(wchar_t symbol)
  * @return the code of the symbol, if @p symbol is multichar; its first (and
  *         only) character otherwise.
  */
-int 
-AttCompiler::symbol_code(const wstring& symbol) 
+int
+AttCompiler::symbol_code(const wstring& symbol)
 {
   if (symbol.length() > 1) {
     alphabet.includeSymbol(symbol);
@@ -111,8 +111,8 @@ AttCompiler::symbol_code(const wstring& symbol)
   }
 }
 
-void 
-AttCompiler::parse(string const &file_name, wstring const &dir) 
+void
+AttCompiler::parse(string const &file_name, wstring const &dir)
 {
   clear();
 
@@ -121,13 +121,14 @@ AttCompiler::parse(string const &file_name, wstring const &dir)
   wstring line;
   bool first_line = true;       // First line -- see below
   bool seen_input_symbol = false;
-  while (getline(infile, line)) 
+  while (getline(infile, line))
   {
     tokens.clear();
     int from, to;
     wstring upper, lower;
+    double weight;
 
-    if (line.length() == 0 && first_line) 
+    if (line.length() == 0 && first_line)
     {
       wcerr << "Error: empty file '" << file_name << "'." << endl;
       exit(EXIT_FAILURE);
@@ -139,30 +140,38 @@ AttCompiler::parse(string const &file_name, wstring const &dir)
     }
 
     /* Empty line. */
-    if (line.length() == 0) 
+    if (line.length() == 0)
     {
       continue;
     }
     split(line, L'\t', tokens);
 
-    from = convert(tokens[0]);
+    from = stoi(tokens[0]);
 
     AttNode* source = get_node(from);
     /* First line: the initial state is of both types. */
-    if (first_line) 
+    if (first_line)
     {
       starting_state = from;
       first_line = false;
     }
 
     /* Final state. */
-    if (tokens.size() <= 2) 
+    if (tokens.size() <= 2)
     {
-      finals.insert(from);
-    } 
-    else 
+      if (tokens.size() > 1)
+      {
+        weight = stod(tokens[1]);
+      }
+      else
+      {
+        weight = default_weight;
+      }
+      finals.insert(pair <int, double>(from, weight));
+    }
+    else
     {
-      to = convert(tokens[1]);
+      to = stoi(tokens[1]);
       if(dir == L"RL")
       {
         upper = tokens[3];
@@ -177,17 +186,24 @@ AttCompiler::parse(string const &file_name, wstring const &dir)
       convert_hfst(lower);
       if(upper != L"")
       {
-        seen_input_symbol = true; 
+        seen_input_symbol = true;
       }
-      /* skip lines that have an empty left side and output 
+      /* skip lines that have an empty left side and output
          if we haven't seen an input symbol */
-      if(upper == L"" && lower != L"" && !seen_input_symbol) 
+      if(upper == L"" && lower != L"" && !seen_input_symbol)
       {
         continue;
       }
       int tag = alphabet(symbol_code(upper), symbol_code(lower));
-      /* We don't read the weights, even if they are defined. */
-      source->transductions.push_back(Transduction(to, upper, lower, tag));
+      if(tokens.size() > 4)
+      {
+        weight = stod(tokens[4]);
+      }
+      else
+      {
+        weight = default_weight;
+      }
+      source->transductions.push_back(Transduction(to, upper, lower, tag, weight));
 
       get_node(to);
     }
@@ -201,8 +217,8 @@ AttCompiler::parse(string const &file_name, wstring const &dir)
 }
 
 /** Extracts the sub-transducer made of states of type @p type. */
-Transducer 
-AttCompiler::extract_transducer(TransducerType type) 
+Transducer
+AttCompiler::extract_transducer(TransducerType type)
 {
   Transducer transducer;
   /* Correlation between the graph's state ids and those in the transducer. */
@@ -214,11 +230,11 @@ AttCompiler::extract_transducer(TransducerType type)
 
   /* The final states. */
   bool noFinals = true;
-  for (set<int>::const_iterator f = finals.begin(); f != finals.end(); ++f) 
+  for (auto& f : finals)
   {
-    if (corr.find(*f) != corr.end()) 
+    if (corr.find(f.first) != corr.end())
     {
-      transducer.setFinal(corr[*f]);
+      transducer.setFinal(corr[f.first], f.second);
       noFinals = false;
     }
   }
@@ -229,9 +245,9 @@ AttCompiler::extract_transducer(TransducerType type)
     wcerr << L"No final states (" << type << ")" << endl;
     wcerr << L"  were:" << endl;
     wcerr << L"\t" ;
-    for (set<int>::const_iterator f = finals.begin(); f != finals.end(); ++f) 
+    for (auto& f : finals)
     {
-      wcerr << *f << L" ";
+      wcerr << f.first << L" ";
     }
     wcerr << endl;
   }
@@ -243,16 +259,16 @@ AttCompiler::extract_transducer(TransducerType type)
  * Recursively fills @p transducer (and @p corr) -- helper method called by
  * extract_transducer().
  */
-void 
+void
 AttCompiler::_extract_transducer(TransducerType type, int from,
-                         Transducer& transducer, map<int, int>& corr,
-                         set<int>& visited) 
+                                 Transducer& transducer, map<int, int>& corr,
+                                 set<int>& visited)
 {
-  if (visited.find(from) != visited.end()) 
+  if (visited.find(from) != visited.end())
   {
     return;
-  } 
-  else 
+  }
+  else
   {
     visited.insert(from);
   }
@@ -263,36 +279,35 @@ AttCompiler::_extract_transducer(TransducerType type, int from,
   bool new_from = corr.find(from) == corr.end();
   int from_t, to_t;
 
-  for (vector<Transduction>::const_iterator it = source->transductions.begin();
-       it != source->transductions.end(); ++it) 
+  for (auto& it : source->transductions)
   {
-    if ((it->type & type) != type) 
+    if ((it.type & type) != type)
     {
       continue;  // Not the right type
     }
     /* Is the target state new? */
-    bool new_to = corr.find(it->to) == corr.end();
+    bool new_to = corr.find(it.to) == corr.end();
 
-    if (new_from) 
+    if (new_from)
     {
       corr[from] = transducer.size() + (new_to ? 1 : 0);
     }
     from_t = corr[from];
 
     /* Now with the target state: */
-    if (!new_to) 
+    if (!new_to)
     {
       /* We already know it, possibly by a different name: link them! */
-      to_t = corr[it->to];
-      transducer.linkStates(from_t, to_t, it->tag);
-    } 
-    else 
+      to_t = corr[it.to];
+      transducer.linkStates(from_t, to_t, it.tag, it.weight);
+    }
+    else
     {
       /* We haven't seen it yet: add a new state! */
-      to_t = transducer.insertNewSingleTransduction(it->tag, from_t);
-      corr[it->to] = to_t;
+      to_t = transducer.insertNewSingleTransduction(it.tag, from_t, it.weight);
+      corr[it.to] = to_t;
     }
-    _extract_transducer(type, it->to, transducer, corr, visited);
+    _extract_transducer(type, it.to, transducer, corr, visited);
   }  // for
 }
 
@@ -310,56 +325,59 @@ AttCompiler::_extract_transducer(TransducerType type, int from,
  * @param visited the ids of states visited by this path.
  * @param path are we in a path?
  */
-void 
+void
 AttCompiler::classify(int from, map<int, TransducerType>& visited, bool path,
-              TransducerType type) 
+                      TransducerType type)
 {
   AttNode* source = get_node(from);
-  if (visited.find(from) != visited.end()) 
+  if (visited.find(from) != visited.end())
   {
-    if (path && ( (visited[from] & type) == type) ) 
+    if (path && ( (visited[from] & type) == type) )
     {
       return;
     }
   }
 
-  if (path) 
+  if (path)
   {
     visited[from] |= type;
   }
 
-  for (vector<Transduction>::iterator it = source->transductions.begin();
-       it != source->transductions.end(); ++it) 
+  for (auto& it : source->transductions)
   {
     bool next_path = path;
     int  next_type = type;
-    bool first_transition = !path && it->upper != L"";
-    if (first_transition) 
+    bool first_transition = !path && it.upper != L"";
+    if (first_transition)
     {
       /* First transition: we now know the type of the path! */
-      bool upper_word  = (it->upper.length() == 1 &&
-                          letters.find(it->upper[0]) != letters.end());
-      bool upper_punct = (it->upper.length() == 1 && iswpunct(it->upper[0]));
+      bool upper_word  = (it.upper.length() == 1 &&
+                          letters.find(it.upper[0]) != letters.end());
+      bool upper_punct = (it.upper.length() == 1 && iswpunct(it.upper[0]));
       next_type = UNDECIDED;
       if (upper_word)  next_type |= WORD;
       if (upper_punct) next_type |= PUNCT;
       next_path = true;
-    } 
-    else 
+    }
+    else
     {
       /* Otherwise (not yet, already): target's type is the same as ours. */
       next_type = type;
     }
-    it->type |= next_type;
-    classify(it->to, visited, next_path, next_type);
+    it.type |= next_type;
+    classify(it.to, visited, next_path, next_type);
   }  // for
 }
 
 /** Writes the transducer to @p file_name in lt binary format. */
-void 
-AttCompiler::write(FILE *output) 
+void
+AttCompiler::write(FILE *output)
 {
 //  FILE* output = fopen(file_name, "w");
+  fwrite(HEADER_LTTOOLBOX, 1, 4, output);
+  uint64_t features = 0;
+  write_le(output, features);
+
   Transducer punct_fst = extract_transducer(PUNCT);
 
   /* Non-multichar symbols. */
@@ -381,7 +399,7 @@ AttCompiler::write(FILE *output)
   wcout << L"main@standard" << " " << word_fst.size();
   wcout << " " << word_fst.numberOfTransitions() << endl;
   Compression::wstring_write(L"final@inconditional", output);
-  if(punct_fst.numberOfTransitions() != 0) 
+  if(punct_fst.numberOfTransitions() != 0)
   {
     punct_fst.write(output);
     wcout << L"final@inconditional" << " " << punct_fst.size();
