@@ -18,16 +18,18 @@
 #include <lttoolbox/my_stdio.h>
 
 #include <cstdlib>
+#include <cmath>
+#include <limits>
 #include <iostream>
 
-void 
+void
 Compression::writeByte(unsigned char byte, FILE *output)
 {
   if(fwrite_unlocked(&byte, 1, 1, output) != 1)
   {
     wcerr << L"I/O Error writing" << endl;
     exit(EXIT_FAILURE);
-  }  
+  }
 }
 
 unsigned char
@@ -65,7 +67,7 @@ Compression::multibyte_write(unsigned int value, FILE *output)
     unsigned char low = (unsigned char) value;
     unsigned char middle = (unsigned char) (value >> 8);
     unsigned char up = (unsigned char) (value >> 16);
-    up = up | 0x80; 
+    up = up | 0x80;
     writeByte(up, output);
     writeByte(middle, output);
     writeByte(low, output);
@@ -76,7 +78,7 @@ Compression::multibyte_write(unsigned int value, FILE *output)
     unsigned char middlelow = (unsigned char) (value >> 8);
     unsigned char middleup = (unsigned char) (value >> 16);
     unsigned char up = (unsigned char) (value >> 24);
-    up = up | 0xc0; 
+    up = up | 0xc0;
     writeByte(up, output);
     writeByte(middleup, output);
     writeByte(middlelow, output);
@@ -110,11 +112,11 @@ Compression::multibyte_write(unsigned int value, ostream &output)
     unsigned char low = (unsigned char) value;
     unsigned char middle = (unsigned char) (value >> 8);
     unsigned char up = (unsigned char) (value >> 16);
-    up = up | 0x80; 
+    up = up | 0x80;
     output.write(reinterpret_cast<char *>(&up), sizeof(char));
     output.write(reinterpret_cast<char *>(&middle), sizeof(char));
     output.write(reinterpret_cast<char *>(&low), sizeof(char));
-    
+
   }
   else if(value < 0x40000000)
   {
@@ -122,7 +124,7 @@ Compression::multibyte_write(unsigned int value, ostream &output)
     unsigned char middlelow = (unsigned char) (value >> 8);
     unsigned char middleup = (unsigned char) (value >> 16);
     unsigned char up = (unsigned char) (value >> 24);
-    up = up | 0xc0; 
+    up = up | 0xc0;
 
     output.write(reinterpret_cast<char *>(&up), sizeof(char));
     output.write(reinterpret_cast<char *>(&middleup), sizeof(char));
@@ -186,7 +188,7 @@ Compression::multibyte_read(FILE *input)
     result = (unsigned int) low;
     result = result | aux;
   }
-   
+
   return result;
 }
 
@@ -246,7 +248,7 @@ Compression::multibyte_read(istream &input)
     result = (unsigned int) low;
     result = result | aux;
   }
-   
+
   return result;
 }
 
@@ -255,9 +257,9 @@ void
 Compression::wstring_write(wstring const &str, FILE *output)
 {
   Compression::multibyte_write(str.size(), output);
-  for(unsigned int i = 0, limit = str.size(); i != limit; i++)
+  for(auto c : str)
   {
-    Compression::multibyte_write(static_cast<int>(str[i]), output);
+    Compression::multibyte_write(static_cast<int>(c), output);
   }
 }
 
@@ -279,9 +281,9 @@ void
 Compression::string_write(string const &str, FILE *output)
 {
   Compression::multibyte_write(str.size(), output);
-  for(unsigned int i = 0, limit = str.size(); i != limit; i++)
+  for(auto c : str)
   {
-    Compression::multibyte_write(static_cast<int>(str[i]), output);
+    Compression::multibyte_write(static_cast<int>(c), output);
   }
 }
 
@@ -295,7 +297,167 @@ Compression::string_read(FILE *input)
   {
     retval += static_cast<char>(Compression::multibyte_read(input));
   }
-  
+
   return retval;
 }
 
+
+void
+Compression::long_multibyte_write(const double& value, FILE *output)
+{
+  int exp = 0;
+
+  unsigned int mantissa = static_cast<unsigned int>(0x40000000 * frexp(value, &exp));
+  unsigned int exponent = static_cast<unsigned int>(exp);
+
+  if(mantissa < 0x04000000)
+  {
+    multibyte_write(mantissa, output);
+  }
+  else
+  {
+    unsigned int low_mantissa = (unsigned int) (mantissa << 6);
+    low_mantissa = (unsigned int) (low_mantissa >> 6);
+    unsigned int up_mantissa = (unsigned int) (mantissa >> 26);
+    up_mantissa = up_mantissa | 0x04000000;
+    multibyte_write(up_mantissa, output);
+    multibyte_write(low_mantissa, output);
+  }
+
+  if(exponent < 0x04000000)
+  {
+    multibyte_write(exponent, output);
+  }
+  else
+  {
+    unsigned int low_exponent = (unsigned int) (exponent << 6);
+    low_exponent = (unsigned int) (low_exponent >> 6);
+    unsigned int up_exponent = (unsigned int) (exponent >> 26);
+    up_exponent = up_exponent | 0x04000000;
+    multibyte_write(up_exponent, output);
+    multibyte_write(low_exponent, output);
+  }
+}
+
+void
+Compression::long_multibyte_write(const double& value, ostream &output)
+{
+  int exp = 0;
+
+  unsigned int mantissa = static_cast<unsigned int>(0x40000000 * frexp(value, &exp));
+  unsigned int exponent = static_cast<unsigned int>(exp);
+
+  if(mantissa < 0x04000000)
+  {
+    multibyte_write(mantissa, output);
+  }
+  else
+  {
+    unsigned int low_mantissa = (unsigned int) (mantissa << 6);
+    low_mantissa = (unsigned int) (low_mantissa >> 6);
+    unsigned int up_mantissa = (unsigned int) (mantissa >> 26);
+    up_mantissa = up_mantissa | 0x04000000;
+    multibyte_write(up_mantissa, output);
+    multibyte_write(low_mantissa, output);
+  }
+
+  if(exponent < 0x04000000)
+  {
+    multibyte_write(exponent, output);
+  }
+  else
+  {
+    unsigned int low_exponent = (unsigned int) (exponent << 6);
+    low_exponent = (unsigned int) (low_exponent >> 6);
+    unsigned int up_exponent = (unsigned int) (exponent >> 26);
+    up_exponent = up_exponent | 0x04000000;
+    multibyte_write(up_exponent, output);
+    multibyte_write(low_exponent, output);
+  }
+}
+
+double
+Compression::long_multibyte_read(FILE *input)
+{
+  double result = 0.0;
+
+  unsigned int mantissa = 0;
+  unsigned int exponent = 0;
+
+  unsigned int up_mantissa = multibyte_read(input);
+  if(up_mantissa < 0x04000000)
+  {
+    mantissa = up_mantissa;
+  }
+  else
+  {
+    up_mantissa = up_mantissa & 0x03ffffff;
+    unsigned int aux = (unsigned int) up_mantissa;
+    aux = aux << 26;
+    unsigned int low_mantissa = multibyte_read(input);
+    mantissa = (unsigned int) (low_mantissa);
+    mantissa = mantissa | aux;
+  }
+  unsigned int up_exponent = multibyte_read(input);
+  if(up_exponent < 0x04000000)
+  {
+    exponent = up_exponent;
+  }
+  else
+  {
+    up_exponent = up_exponent & 0x03ffffff;
+    unsigned int aux = (unsigned int) up_exponent;
+    aux = aux << 26;
+    unsigned int low_exponent = multibyte_read(input);
+    exponent = (unsigned int) (low_exponent);
+    exponent = exponent | aux;
+  }
+
+  double value = static_cast<double>(static_cast<int>(mantissa)) / 0x40000000;
+  result = ldexp(value, static_cast<int>(exponent));
+
+  return result;
+}
+
+double
+Compression::long_multibyte_read(istream &input)
+{
+  double result = 0.0;
+
+  unsigned int mantissa = 0;
+  unsigned int exponent = 0;
+
+  unsigned int up_mantissa = multibyte_read(input);
+  if(up_mantissa < 0x04000000)
+  {
+    mantissa = up_mantissa;
+  }
+  else
+  {
+    up_mantissa = up_mantissa & 0x03ffffff;
+    unsigned int aux = (unsigned int) up_mantissa;
+    aux = aux << 26;
+    unsigned int low_mantissa = multibyte_read(input);
+    mantissa = (unsigned int) (low_mantissa);
+    mantissa = mantissa | aux;
+  }
+  unsigned int up_exponent = multibyte_read(input);
+  if(up_exponent < 0x04000000)
+  {
+    exponent = up_exponent;
+  }
+  else
+  {
+    up_exponent = up_exponent & 0x03ffffff;
+    unsigned int aux = (unsigned int) up_exponent;
+    aux = aux << 26;
+    unsigned int low_exponent = multibyte_read(input);
+    exponent = (unsigned int) (low_exponent);
+    exponent = exponent | aux;
+  }
+
+  double value = static_cast<double>(static_cast<int>(mantissa)) / 0x40000000;
+  result = ldexp(value, static_cast<int>(exponent));
+
+  return result;
+}
