@@ -44,6 +44,22 @@ read_fst(FILE *bin_file)
 
   std::map<wstring, Transducer> transducers;
 
+  fpos_t pos;
+  if (fgetpos(bin_file, &pos) == 0) {
+      char header[4]{};
+      fread(header, 1, 4, bin_file);
+      if (strncmp(header, HEADER_LTTOOLBOX, 4) == 0) {
+          auto features = read_le<uint64_t>(bin_file);
+          if (features >= LTF_UNKNOWN) {
+              throw std::runtime_error("FST has features that are unknown to this version of lttoolbox - upgrade!");
+          }
+      }
+      else {
+          // Old binary format
+          fsetpos(bin_file, &pos);
+      }
+  }
+
   // letters
   int len = Compression::multibyte_read(bin_file);
   while(len > 0)
