@@ -1,16 +1,16 @@
-#!/usr/bin/env python2
+#!/usr/bin/python2
 
 import sys
 import os
 sys.path.append(os.path.realpath("."))
 
 import unittest
-import lt_proc, lt_trim
+import lt_proc, lt_trim, lt_print
 
 if __name__ == "__main__":
     os.chdir(os.path.dirname(__file__))
     failures = 0
-    for module in [lt_trim, lt_proc]:
+    for module in [lt_trim, lt_proc, lt_print]:
         suite = unittest.TestLoader().loadTestsFromModule(module)
         res = unittest.TextTestRunner(verbosity = 2).run(suite)
         failures += len(res.failures)
